package ij2x.gui;
import ij.*;
import ij.gui.*;
import ij.util.*;
import java.awt.*;
import java.util.Vector;
import ij.plugin.frame.PlugInFrame;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import javax.swing.border.Border;
import javax.swing.border.BevelBorder;


 /** Displays a dialog that allows the user to select a color using three sliders. */
public class JColorChooser extends PlugInFrame implements DocumentListener, ChangeListener {
    Vector<JComponent> colors, sliders, spinners;
    JColorPanel panel;
    Color initialColor=new Color(255, 255, 0);
    int red, green, blue;
    boolean useHSB;
    String title;
    boolean mono;
    private Document[] colorsDoc=new Document[3];
    private JSlider[] slider=new JSlider[3] ;
    private JSpinner[] spinner=new JSpinner[3];
    private String[] rgbColor=new String[3];
    private int[] rgbcolor=new int[3];
    private BevelBorder bb ;//new BevelBorder for LOWER or RAISED type
    private Border grayBorder = BorderFactory.createBevelBorder(bb.RAISED, Color.gray, Color.lightGray);
    private Border emptyBorder = BorderFactory.createEmptyBorder(2,2,2,2);
    private Font font = new Font("SansSerif",Font.BOLD,12);
    private Container contentPane;
    static final String LOC_KEY = "cc.loc";
    static JColorChooser instance;

    /** Constructs a ColorChooser using the specified title and initial color. */
    public JColorChooser(String title, Color initialColor, boolean useHSB) {
        super(title);
        this.title = title;
        this.initialColor = initialColor;
        if (instance!=null) {
                WindowManager.toFront(instance);
                return;
        }
        instance = this;
        WindowManager.addWindow(this);
        red = this.initialColor.getRed();
        green = this.initialColor.getGreen();
        blue = this.initialColor.getBlue();
        mono = red==green && green==blue;
        this.useHSB = useHSB;
    }

    /** Displays a color selection dialog and returns the color selected by the user. */
    public Color getColor() {
        GenericDialog gd = new GenericDialog(title);
        //set slider values
        String redLabel="Red:", greenLabel="Green:", blueLabel="Blue:";
        int min=0, max=255;
        int redvalue=red, greenValue=green, blueValue=blue;
        int mior=17;
        int major=255/3;
        int width=164, height=58;
        boolean paintticks=true, paintLabels=true;
        JLabel rgbLabel = new JLabel(" RGB Color ");
        JPanel rgbPanel = new JPanel();
        rgbLabel.setBorder(grayBorder);
        rgbPanel.add(rgbLabel);
        gd.addPanel(rgbPanel, GridBagConstraints.WEST, new Insets(0, 0, 0, 10));
        gd.addJSlider(redLabel, min, max, redvalue, major, mior, paintticks, paintLabels, width, height);
        gd.addJSlider(greenLabel, min, max, greenValue, major, mior, paintticks, paintLabels, width, height);
        gd.addJSlider(blueLabel, min, max, blueValue, major, mior, paintticks, paintLabels, width, height);
        JLabel theLabel = (JLabel)gd.getMessage();
        panel = new JColorPanel(initialColor);
        int left = theLabel.getPreferredSize().width;
        gd.addPanel(panel, GridBagConstraints.CENTER, new Insets(10, 0, 10, 0));
        colors = gd.getSpinnerFields();
        for (int i=0; i<colors.size(); i++){
            colorsDoc[i] = ((JFormattedTextField) colors.elementAt(i)).getDocument();
            colorsDoc[i].addDocumentListener(this);
        }
        sliders = gd.getSliders();
        spinners=gd.getSpinners();
        for (int i=0; i<sliders.size(); i++){
            //((JScrollBar)sliders.elementAt(i)).addAdjustmentListener(this);
            spinner[i]=(JSpinner) spinners.elementAt(i);
            slider[i]=(JSlider) sliders.elementAt(i);
            spinner[i].addChangeListener(this);
            slider[i].addChangeListener(this);
            //((JSpinner) spinners.elementAt(i)).addChangeListener(this);
            //((JSlider) sliders.elementAt(i)).addChangeListener(this);
        }
        contentPane=gd.getContentPane();
        container.add(contentPane);
        ImageJ ij = IJ.getInstance();
        pack();
        Point loc = Prefs.getLocation(LOC_KEY);
        if (loc!=null)
                setLocation(loc);
        else
                GUI.relRight(ij, this);
        setVisible(true);
        //gd.showDialog();
        //if (gd.wasCanceled()) return initialColor;
        int red = (int)gd.getNextNumberS();
        int green = (int)gd.getNextNumberS();
        int blue = (int)gd.getNextNumberS();
        return new Color(red, green, blue);
    }

    void textFieldUpdate(DocumentEvent e){
        try {
            for(int i=0; i < rgbColor.length; i++){
                rgbColor[i] = colorsDoc[i].getText(0, colorsDoc[i].getLength());
                rgbcolor[i] = (int) Tools.parseDouble(rgbColor[i]);
            }
            red = rgbcolor[0];
            green = rgbcolor[1];
            blue = rgbcolor[2];
            if (red<0) red=0; if (red>255) red=255;
            if (green<0) green=0; if (green>255) green=255;
            if (blue<0) blue=0; if (blue>255) blue=255;
            rgbcolor[0]=red;
            rgbcolor[1]=green;
            rgbcolor[2]=blue;
            for(int i=0; i< slider.length; i++){
                slider[i].setValue(rgbcolor[i]);
                spinner[i].setValue(Integer.toString(rgbcolor[i]));
            }
        } catch (Exception ex) {

        }

        if (Toolbar.isForegroundColor){
            Toolbar.setForegroundColor(new Color(red, green, blue));
            panel.setColor(Toolbar.getForegroundColor());
        }
        else{
            Toolbar.setBackgroundColor(new Color(red, green, blue));
            panel.setColor(Toolbar.getBackgroundColor());
        }
       panel.repaint();
   }
   public void close() {
               super.close();
               instance = null;
               Prefs.saveLocation(LOC_KEY, getLocation());
               IJ.notifyEventListeners(IJEventListener.COLOR_CHOICER_CLOSED);
       }

    public void insertUpdate(DocumentEvent e){
        textFieldUpdate(e);
    }
    public void removeUpdate(DocumentEvent e){
        textFieldUpdate(e);
    }
    public void changedUpdate(DocumentEvent e){
    }

    public synchronized void stateChanged(ChangeEvent e) {
            Object source = e.getSource();
            /*for (int i=0; i<sliders.size(); i++) {
                if (source==sliders.elementAt(i)) {
                            JSlider sb = (JSlider)source;
                            JFormattedTextField tf = (JFormattedTextField)colors.elementAt(i);
                            if (i==0 && mono) { // red
                                    String red = tf.getText();
                                    JFormattedTextField tf1 = (JFormattedTextField)colors.elementAt(1);
                                    JFormattedTextField tf2 = (JFormattedTextField)colors.elementAt(2);
                                    tf1.setText(red);
                                    tf2.setText(red);
                            }
                            if (i!=0) mono = false;
                    }
            }*/
    }

}

class JColorPanel extends JPanel {
    Color c;
    static String name = ImageJ.PRODUCT_NAME+" "+ImageJ.VERSION;
    static int length = 180;
    static final int WIDTH=length, HEIGHT=50;
    JColorPanel(Color c) {
        this.c = c;
    }

    public Dimension getPreferredSize() {
        return new Dimension(WIDTH, HEIGHT);
    }

    void setColor(Color c) {
        this.c = c;
    }

    public Dimension getMinimumSize() {
        return new Dimension(WIDTH, HEIGHT);
    }
    public void update(Graphics g) {
       paintComponent(g);
    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2D = (Graphics2D)g;
        g2D.setColor(c);
        g2D.fillRect(0, 0, WIDTH/8, HEIGHT);
        g2D.setColor(Color.black);
        g2D.fillRect(WIDTH/8, 0, WIDTH/8, HEIGHT);
        g2D.setColor(c);
        g2D.fillRect((WIDTH/4)+2, 0, (WIDTH/4+WIDTH/2), HEIGHT);
        //g.setFont(font);
        g2D.setXORMode(g2D.getBackground());
        g2D.fillRect((WIDTH/4)+2+4, 4, (WIDTH/4+WIDTH/2)-10, HEIGHT-8);
        g2D.setColor(c);
        g2D.drawString(name, WIDTH/4+10, HEIGHT/3);
        g2D.setColor(Color.black);
        g2D.drawString(name, WIDTH/4+10, HEIGHT/2+15);
        //g.draw3DRect(-1, -1, WIDTH+2, HEIGHT+2, true);
    }

}
