package ij2x.plugin;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.util.MissingResourceException;
import java.util.Locale;
import java.util.ResourceBundle;

import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.Icon;
import javax.swing.border.BevelBorder;
import javax.swing.BorderFactory;
import javax.swing.border.Border;

import ij.IJ;
import ij.ImageJ;
import ij.ImagePlus;
import ij.Prefs;
import ij.WindowManager;
import ij.util.Tools;
import ij.plugin.PlugIn;
import ij.plugin.frame.Editor;

import ij2x.plugin.imageborder.ImageBorderPanel;
import ij2x.util.ResourceString;
import ij2x.util.BorderlessButton;
import ij2x.util.AWTImages;
import ij2x.util.ImageIcons;


/**
 * <p>Title: </p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2012</p>
 *
 * <p>Company: </p>
 *
 * @author not attributable
 * @version 1.0
 */
public class LutPanel extends JPanel implements PlugIn, ActionListener, MouseListener, MouseMotionListener {

    protected ImagePlus imp;
    protected ImageJ ij;
    public JButton infoButton, draggedButton, closeButton, undecoratedButton;
    protected JButton newButton, readLutButton, exportLutButton, redLutButton, greenLutButton, blueLutButton, lutSlideShowButton, showLutSlideButton
            , grayLutButton, saveAsJavaButton, rgbButton, gradientButton, alignButton, mergergbButton, thresholdButton, showlutButton, editlutButton, editButton;
    private JButton action;
    private BevelBorder bb ;//new BevelBorder for LOWER or RAISED type
    private Border lightGrayBorder = BorderFactory.createBevelBorder(bb.RAISED, Color.lightGray, Color.lightGray);
    private int actionCount = 0;
    private static int current;
    private long mouseDownTime;
    public static final int DOUBLE_CLICK_THRESHOLD = 150;
    public JPanel controlBarPanel=new JPanel();
    public JPanel panel=new JPanel();
    private GridBagLayout gblayout = new GridBagLayout();
    private boolean previousOrNext = false;

    private static ResourceBundle resources;
    private int index = 0;

    static {
        try {
            resources = ResourceBundle.getBundle("ij2x.plugin.resources.LutBar",
                                                 Locale.getDefault());
        } catch (MissingResourceException mre) {
            System.err.println("resources/LutBar.properties not found");
            System.exit(1);
        }
    }

    public LutPanel() {
    }
    public void run(String arg){
        ij = IJ.getInstance();
        controlBarPanel=createControlBar();
    }

    public JPanel createControlBar(){

        java.awt.FlowLayout flayout = new java.awt.FlowLayout(java.awt.FlowLayout.LEFT);
        flayout.setHgap(0);
        flayout.setVgap(0);
        panel.setLayout(flayout);
        panel.setBorder(lightGrayBorder);
        ImageIcon b1o = ImageIcons.getIcons(this);

        AWTImages.setImageDir("lutDir");


    String[] iconAllName = ResourceString.tokenize(ResourceString.getResourceString(resources, "lutbar"));

    Image[][] icons = new Image[iconAllName.length][3];
    for(int i=0; i < iconAllName.length; i++){
        for(int j=0; j < 3; j++){
            icons[i][j]=((ImageIcon)AWTImages.createBufferedIcon(
                    this
                      , ResourceString.tokenize(ResourceString.getResourceString(resources, iconAllName[i]))[j]
                        , b1o.getImage()
                          , 24)).getImage();
        }
    }

    newButton=BorderlessButton.createButton(this, newButton, "New Image", icons[index++]);

    readLutButton=BorderlessButton.createButton(this, readLutButton, "Open Lut", icons[index++]);

    exportLutButton=BorderlessButton.createButton(this, exportLutButton, "Export Lut", icons[index++]);

    rgbButton=BorderlessButton.createButton(this, rgbButton, "RGB Split", icons[index++]);

    mergergbButton=BorderlessButton.createButton(this, mergergbButton, "RGB Merge", icons[index++]);

    thresholdButton=BorderlessButton.createButton(this, thresholdButton, "Threshold", icons[index++]);

    redLutButton=BorderlessButton.createButton(this, redLutButton, "Red Lut", icons[index++]);

    greenLutButton=BorderlessButton.createButton(this, greenLutButton, "Green Lut", icons[index++]);

    blueLutButton=BorderlessButton.createButton(this, blueLutButton, "Blue Lut", icons[index++]);

    grayLutButton=BorderlessButton.createButton(this, grayLutButton, "Gray Lut", icons[index++]);

    saveAsJavaButton=BorderlessButton.createButton(this, saveAsJavaButton, "Save As Java", icons[index++]);

    gradientButton=BorderlessButton.createButton(this, gradientButton, "Gradient", icons[index++]);

    lutSlideShowButton=BorderlessButton.createButton(this, lutSlideShowButton, "LUT Slide Show Left", icons[index++]);

    showLutSlideButton=BorderlessButton.createButton(this, showLutSlideButton, "LUT Slide Show Right", icons[index++]);

    alignButton=BorderlessButton.createButton(this, alignButton, "Align RGB", icons[index++]);

    showlutButton=BorderlessButton.createButton(this, showlutButton, "Show LUT", icons[index++]);

    editlutButton=BorderlessButton.createButton(this, editlutButton, "Edit LUT", icons[index++]);

    editButton=BorderlessButton.createButton(this, editButton, "Edit Java", icons[index++]);

    infoButton=BorderlessButton.createButton(this, infoButton, "Show Info ...", icons[index]);

        panel.add(newButton);
        panel.add(readLutButton);
        panel.add(rgbButton);
        panel.add(thresholdButton);
        panel.add(mergergbButton);
        panel.add(redLutButton);
        panel.add(greenLutButton);
        panel.add(blueLutButton);
        panel.add(grayLutButton);
        panel.add(lutSlideShowButton);
        panel.add(showLutSlideButton);
        panel.add(editlutButton);
        panel.add(showlutButton);
        panel.add(gradientButton);
        panel.add(alignButton);
        panel.add(editButton);
        panel.add(saveAsJavaButton);
        panel.add(infoButton);
        panel.add(exportLutButton);

        return panel;
     }

       public void mousePressed(MouseEvent e) {
           action = (JButton)e.getSource();
           int x = e.getX();
           boolean doubleClick = (System.currentTimeMillis()-mouseDownTime)<=DOUBLE_CLICK_THRESHOLD;
           mouseDownTime = System.currentTimeMillis();
                if (!doubleClick) {
                }
                if (e.isMetaDown())previousOrNext=false;
                else previousOrNext=true;
                if(action == lutSlideShowButton && !previousOrNext){
                    //IJ.runPlugIn("ij.plugin.LutLoader", "SlideShow LUT");
                }
        }

        public void mouseReleased(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {}

        public void mouseDragged(MouseEvent e) {}
        public void mouseMoved(MouseEvent e) {
            JButton action = (JButton)e.getSource();
            int x = e.getX();
            boolean doubleClick = (System.currentTimeMillis()-mouseDownTime)<=DOUBLE_CLICK_THRESHOLD;
            mouseDownTime = System.currentTimeMillis();
                 if (!false) {
                     if(action == infoButton){
                         IJ.showStatus("Command: Show Info...");
                     }
                     else if(action == readLutButton){
                         IJ.showStatus("Command: Import LUT");
                     }
                     else if(action == newButton){
                         IJ.showStatus("Command: Image...");
                     }
                     else if(action == exportLutButton){
                         IJ.showStatus("Command: Export LUT");
                     }
                     else if(action == redLutButton){
                         IJ.showStatus("Command: Red LUT");
                     }
                     else if(action == greenLutButton){
                         IJ.showStatus("Command: Green LUT");
                     }
                     else if(action == blueLutButton){
                         IJ.showStatus("Command: Blue LUT");
                     }
                     else if(action == grayLutButton){
                         IJ.showStatus("Command: Gray LUT");
                     }
                     else if(action == saveAsJavaButton){
                         IJ.showStatus("Command: Save As Java");
                     }
                     else if(action == editlutButton){
                         IJ.showStatus("Command: Edit LUT...");
                     }
                     else if(action == editButton){
                         IJ.showStatus("Command: Edit...");
                     }
                     else if(action == showlutButton){
                         IJ.showStatus("Command: Show LUT");
                     }
                     else if(action == rgbButton){
                         IJ.showStatus("Command: Split RGB");
                     }
                     else if(action == mergergbButton){
                         IJ.showStatus("Command: RGB Merge...");
                     }
                     else if (action == draggedButton) {
                         IJ.showStatus("Move...");
                     }
                     else if (action == closeButton) {
                         IJ.showStatus("Decorated Frame");
                     }
                     else if (action == undecoratedButton) {
                         IJ.showStatus("Undecorated Frame");
                     }
                     else if(action == gradientButton){
                         IJ.showStatus("Command: Gradient...");
                     }
                     else if(action == lutSlideShowButton){
                         IJ.showStatus("Command: LUT SlideShow");
                     }
                     else if(action == showLutSlideButton){
                         IJ.showStatus("Command: SlideShow LUT");
                     }
                     else if(action == alignButton){
                         IJ.showStatus("Command: AlignRGB...");
                     }
                     else if(action == thresholdButton){
                         IJ.showStatus("Command: Thresold...");
                     }

                 }
        }
        public void actionPerformed(ActionEvent e){
            try {
                if(action == infoButton){
                    IJ.doCommand("Show Info...");
                }
                else if(action == readLutButton){
                    IJ.runPlugIn("ij.plugin.LutLoader", "");
                }
                else if(action == newButton){
                    IJ.doCommand("Image...");
                    //IJ.run("Image...", "");
                }
                else if(action == exportLutButton){
                    IJ.runPlugIn("ij.plugin.filter.Writer", "lut");
                }
                else if(action == redLutButton){
                    IJ.runPlugIn("ij.plugin.LutLoader", "red");
                }
                else if(action == greenLutButton){
                    IJ.runPlugIn("ij.plugin.LutLoader", "green");
                }
                else if(action == blueLutButton){
                    IJ.runPlugIn("ij.plugin.LutLoader", "blue");
                }
                else if(action == grayLutButton){
                    IJ.runPlugIn("ij.plugin.LutLoader", "grays");
                }
                else if(action == saveAsJavaButton){
                    IJ.runPlugIn("ij2x.plugin.Lut2Java", "");
                }
                else if(action == editButton){
                    ImageJ ij = IJ.getInstance();
                    if(Editor.focusedComponent == null){
                        IJ.runPlugIn("ij.plugin.Compiler", "edit");
                        ////Editor ed = (Editor)IJ.runPlugIn("ij.plugin.frame.Editor", "");
                        //ed.create("Untitled.java", "");
                       // IJ.runPlugIn("ij.plugin.NewPlugin", "");
                    }else Editor.focusedComponent.toFront();
                }
                else if(action == editlutButton){
                    IJ.runPlugIn("ij.plugin.LUT_Editor", "");
                }
                else if(action == showlutButton){
                    IJ.runPlugIn("ij2x.plugin.filter.LutViewer", "");
                }
                else if(action == rgbButton){
                    IJ.runPlugIn("ij.plugin.filter.RGBStackSplitter", "is8bit");
                }
                else if(action == mergergbButton){
                    IJ.runPlugIn("ij.plugin.RGBStackMerge", "");
                }
                else if(action == gradientButton){
                    IJ.runPlugIn("ij2x.plugin.HorizontalGradient", "isLUT");
                }
                else if(action == lutSlideShowButton ){
                    //IJ.runPlugIn("ij.plugin.LutLoader", "LUT SlideShow");
                    IJ.runPlugIn("ij2x.plugin.LutScroller", "LUT SlideShow");
                }
                else if(action == showLutSlideButton ){
                    //IJ.runPlugIn("ij.plugin.LutLoader", "SlideShow LUT");
                    IJ.runPlugIn("ij2x.plugin.LutScroller", "SlideShow LUT");
                }
                else if(action == alignButton){
                    IJ.runPlugIn("ij2x.plugin.AlignRGB", "");
                }
                else if(action == thresholdButton){
                    IJ.runPlugIn("ij.plugin.frame.ThresholdAdjuster", "");
                }
            } catch (Exception ex) {

            }
        }
}
