package ij2x.plugin.filter;


import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.image.ColorModel;
import java.awt.image.IndexColorModel;
import javax.swing.JButton;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;

import ij.IJ;
import ij.ImageJ;
import ij.ImagePlus;
import ij.WindowManager;
import ij.process.ByteProcessor;
import ij.process.ImageProcessor;
import ij.process.ImageStatistics;
import ij.gui.GUI;
import ij.measure.Measurements;
import ij.plugin.frame.Recorder;
import ij.plugin.frame.PlugInFrame;
import ij.plugin.frame.PasteController;
import ij.text.TextWindow;

/** Adjusts the lower and upper threshold levels of the active image. This
        class is multi-threaded to provide a more responsive user interface. */
public class LutViewer extends PlugInFrame implements Measurements,
        Runnable, ActionListener {

        private static final double defaultMinThreshold = 85;
        private static final double defaultMaxThreshold = 170;
        private static JFrame instance;
        private ViewerPlot plot = new ViewerPlot();
        private Thread thread;
        private BevelBorder bb ;//new BevelBorder for LOWER or RAISED type
        private Border lightGrayBorder = BorderFactory.createBevelBorder(bb.RAISED, Color.lightGray, Color.lightGray);

        private JButton button;
        private int previousImageID;
        private int previousImageType;
        private double previousMin, previousMax;
        private ImageJ ij;
        private double minThreshold, maxThreshold;  // 0-255
        private boolean done;
        private boolean invertedLut;
        private boolean firstActivation;
        private ImagePlus imp;

        public LutViewer() {
                super("LutViewer");
                if (instance!=null) {
                        instance.toFront();
                        return;
                }

                WindowManager.addWindow(this);
                instance = this;
                IJ.register(PasteController.class);

                ij = IJ.getInstance();

                imp = WindowManager.getCurrentImage();

                Font font = new Font("SansSerif", Font.PLAIN, 10);
                GridBagLayout gridbag = new GridBagLayout();
                GridBagConstraints c = new GridBagConstraints();
                container.setLayout(gridbag);
                this.getRootPane().setBorder(lightGrayBorder);

                // plot
                int y = 0;
                c.gridx = 0;
                c.gridy = y++;
                c.gridwidth = 2;
                c.fill = GridBagConstraints.BOTH;
                c.anchor = GridBagConstraints.CENTER;
                c.insets = new Insets(10, 10, 0, 10);
                container.add(plot, c);
                plot.addKeyListener(ij);

                addPanel(c, y);

                addKeyListener(ij);  // ImageJ handles keyboard shortcuts
                pack();
                GUI.center(this);
                firstActivation = true;
                if (IJ.isMacOSX()) setResizable(false);
                setVisible(true);

                thread = new Thread(this, "LutViewer");
                //thread.setPriority(thread.getPriority()-1);
                thread.start();
                if (imp!=null){
                    setup(imp);
                }
        }

        void addPanel(GridBagConstraints c, int y) {
            c.gridx = 0;
            c.gridy = y++;
            c.gridwidth = 1;
            c.weightx = 100;
            c.weightx = IJ.isMacintosh()?90:100;
            c.fill = GridBagConstraints.HORIZONTAL;
            c.insets = new Insets(5, 10, 5, 10);
                JPanel panel = new JPanel();
                panel.setLayout(new FlowLayout(FlowLayout.LEFT));
                button = new JButton("List");
                button.addActionListener(this);
                panel.add(button);
                panel.setBorder(lightGrayBorder);
                container.add(panel, c);
                pack();
        }

        void list(ImageProcessor ip) {
                if(ip == null)imp.getProcessor();
                IndexColorModel icm = (IndexColorModel)ip.getColorModel();
                int size = icm.getMapSize();
                byte[] r = new byte[size];
                byte[] g = new byte[size];
                byte[] b = new byte[size];
                icm.getReds(r);
                icm.getGreens(g);
                icm.getBlues(b);
                StringBuffer sb = new StringBuffer();
                String headings = "Index\tRed\tGreen\tBlue";
                for (int i=0; i<size; i++)
                        sb.append(i+"\t"+(r[i]&255)+"\t"+(g[i]&255)+"\t"+(b[i]&255)+"\n");
                TextWindow tw = new TextWindow("LUT", headings, sb.toString(), 250, 400);
        }


        public synchronized  void actionPerformed(ActionEvent e) {
            Object b = e.getSource();
            //imp = WindowManager.getCurrentImage();
            if (b==button && imp != null)
                    list(imp.getProcessor());
        }

        ImageProcessor setup(ImagePlus imp) {
                ImageProcessor ip;
                int type = imp.getType();
                if (type==ImagePlus.COLOR_RGB)
                        return null;
                ip = imp.getProcessor();
                boolean minMaxChange = false;
        boolean not8Bits = type==ImagePlus.GRAY16 || type==ImagePlus.GRAY32;
                if (not8Bits) {
                        if (ip.getMin()!=previousMin || ip.getMax()!=previousMax)
                                minMaxChange = true;
                        previousMin = ip.getMin();
                        previousMax = ip.getMax();
                }
                int id = imp.getID();
                if (minMaxChange || id!=previousImageID || type!=previousImageType) {
            //IJ.log(minMaxChange +"  "+ (id!=previousImageID)+"  "+(type!=previousImageType));
            if (not8Bits && minMaxChange) {
                ip.resetMinAndMax();
                imp.updateAndDraw();
            }
                        invertedLut = imp.isInvertedLut();
                        minThreshold = ip.getMinThreshold();
                        maxThreshold = ip.getMaxThreshold();
                        ImageStatistics stats = plot.setHistogram(imp);
                        if (minThreshold==ip.NO_THRESHOLD)
                                autoSetLevels(ip, stats);
                        updatePlot();
                        imp.updateAndDraw();
                }
                previousImageID = id;
                previousImageType = type;
                return ip;
        }

        void autoSetLevels(ImageProcessor ip, ImageStatistics stats) {
                if (stats==null || stats.histogram==null) {
                        minThreshold = defaultMinThreshold;
                        maxThreshold = defaultMaxThreshold;
                        return;
                }
                int threshold = ip.getAutoThreshold(stats.histogram);
                //IJ.log(threshold+" "+stats.min+" "+stats.max+" "+stats.dmode);
                if ((stats.max-stats.dmode)>(stats.dmode-stats.min)) {
                        minThreshold = threshold;
                        maxThreshold = stats.max;
                } else {
                        minThreshold = stats.min;
                        maxThreshold = threshold;
                }
                if (Recorder.record)
                        Recorder.record("setAutoThreshold");
        }

        void updatePlot() {
                plot.minThreshold = minThreshold;
                plot.maxThreshold = maxThreshold;
                //plot.mode = mode;
                plot.repaint();
        }

        // Separate thread that does the potentially time-consuming processing
        public void run() {
                while (!done) {
                        synchronized(this) {
                                try {wait();}
                                catch(InterruptedException e) {}
                        }
                        doUpdate();
                }
        }

        void doUpdate() {
        }

    public void windowClosing(WindowEvent e) {
        close();
        }

    /** Overrides close() in PlugInFrame. */
    public void close() {
        super.close();
                instance = null;
                done = true;
                synchronized(this) {
                        notify();
                }
        }

    public void windowActivated(WindowEvent e) {
        super.windowActivated(e);
                ImagePlus imp = WindowManager.getCurrentImage();
                if (imp!=null) {
                        if (!firstActivation) {
                                previousImageID = 0;
                                setup(imp);
                        }
                        firstActivation = false;
                }
        }

} // ThresholdAdjuster class


class ViewerPlot extends JComponent implements Measurements, MouseListener {

        static final int WIDTH = 256, HEIGHT=68;
        double minThreshold = 85;
        double maxThreshold = 170;
        int[] histogram;
        Color[] hColors;
        int hmax;
        Image os;
        Graphics osg;
        int mode;

        public ViewerPlot() {
                addMouseListener(this);
                setSize(WIDTH+1, HEIGHT+1);
        }

    /** Overrides Component getPreferredSize(). Added to work
        around a bug in Java 1.4.1 on Mac OS X.*/
    public Dimension getPreferredSize() {
        return new Dimension(WIDTH+1, HEIGHT+1);
    }

        ImageStatistics setHistogram(ImagePlus imp) {
                ImageProcessor ip = imp.getProcessor();
                if (!(ip instanceof ByteProcessor)) {
                        double min = ip.getMin();
                        double max = ip.getMax();
                        ip.setMinAndMax(min, max);
                        ip = new ByteProcessor(ip.createImage());
                }
                ip.setRoi(imp.getRoi());
                ImageStatistics stats = ImageStatistics.getStatistics(ip, AREA+MIN_MAX+MODE, null);
                int maxCount2 = 0;
                histogram = stats.histogram;
                for (int i = 0; i < stats.nBins; i++)
                        if ((histogram[i] > maxCount2) && (i != stats.mode))
                                maxCount2 = histogram[i];
                hmax = stats.maxCount;
                if ((hmax>(maxCount2 * 2)) && (maxCount2 != 0)) {
                        hmax = (int)(maxCount2 * 1.5);
                        histogram[stats.mode] = hmax;
                }
                os = null;

                ColorModel cm = ip.getColorModel();
                if (!(cm instanceof IndexColorModel))
                        return null;
                IndexColorModel icm = (IndexColorModel)cm;
                int mapSize = icm.getMapSize();
                if (mapSize!=256)
                        return null;
                byte[] r = new byte[256];
                byte[] g = new byte[256];
                byte[] b = new byte[256];
                icm.getReds(r);
                icm.getGreens(g);
                icm.getBlues(b);
                hColors = new Color[256];
                for (int i=0; i<256; i++)
                        hColors[i] = new Color(r[i]&255, g[i]&255, b[i]&255);
                return stats;
        }

        public void update(Graphics g) {
            //paint(g);
            paintComponent(g);
        }

        public void paintComponent(Graphics g) {
            super.paintComponent(g);
                if (g==null) return;
                if (histogram!=null) {
                        if (os==null && hmax>0) {
                                os = createImage(WIDTH,HEIGHT);
                                osg = os.getGraphics();
                                UIDefaults uid = UIManager.getLookAndFeel().getDefaults();
                                Color bg = uid.getColor("Panel.background");
                                osg.setColor(Color.white);
                                osg.fillRect(0, 0, WIDTH, HEIGHT-20);
                                osg.setColor(bg);
                                osg.fillRect(0, HEIGHT-20, WIDTH, 20);
                                osg.setColor(Color.gray);
                                for (int i = 0; i < WIDTH; i++) {
                                        if (hColors!=null) osg.setColor(hColors[i]);
                                        osg.drawLine(i, HEIGHT-20, i, HEIGHT-20 - ((int)((HEIGHT-20) * histogram[i])/hmax));
                                        osg.drawLine(i, HEIGHT-12, i, HEIGHT-5);
                                }
                                osg.dispose();
                        }
                        g.drawImage(os, 0, 0, this);
                } else {
                        g.setColor(Color.white);
                        g.fillRect(0, 0, WIDTH, HEIGHT);
                }
                g.setColor(Color.black);
                g.drawRect(0, 0, WIDTH, HEIGHT);
    }

        public void mousePressed(MouseEvent e) {}
        public void mouseReleased(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {}

} // ThresholdPlot class
